#include <TFile.h>
#include <TH2F.h>
#include <TTree.h>
#include <TLeaf.h>
#include <TChain.h>
#include <TString.h>
#include <TGraphErrors.h>
#include <TGraph.h>
#include <TCanvas.h>
#include <TLegend.h>
#include <TROOT.h>
#include <TMinuit.h>

#include <iostream>

using namespace std;

//Set the cut values
double Cut1 = 0.7;
double Cut2 = 0.7;
double Cut3 = 48;
double Cut4 = 250;
double Cut5 = 620;

//Define class Event
class EventC {
public:
  EventC() { m_w = 1; m_fs = 1; m_fb = 0; m_cat = 1; }
  EventC(double w0, double fs0, double fb0) { m_w = w0; m_fs = fs0; m_fb = fb0; }
  EventC(double w0, int cat0) { m_w = w0; m_cat = cat0; m_fs = 1; m_fb = 0; }
  double w() { return m_w; }
  double fs() { return m_fs; }
  double fb() { return m_fb; }
  int cat() { return m_cat; }
  void fs(double fs0) { m_fs = fs0; }
  void fb(double fb0) { m_fb = fb0; }

private:
  double m_w, m_fs, m_fb;
  int m_cat;
};

//global variables
std::vector<EventC> m_EventList;
double m_F1[3][31][6];
double m_F0[3][31][6];
double m_N1exp[3][31];
double m_N0exp[3][31];

void fcnN0N1(Int_t &npar, Double_t *gin, Double_t &f, Double_t *par, Int_t iflag) {
  double L = 0;
  for (unsigned int iev = 0; iev < m_EventList.size(); iev++) {
    EventC evt = m_EventList[iev];
    double smt = par[0]*evt.fb() + par[1]*evt.fs();
    if (smt > 0) L += evt.w()*log(smt);
  }
  f = par[0] + par[1] - L; 
};

vector<double> vFe(double cos1,double cos2,double dMLep, int i, int j)
{
  double f0e = 0;
  double f1e = 0;
  
  //cout << "cos1 = " << cos1 << " cos2 = " << cos2 << " dMLep = " << dMLep << endl;
  
  int it = -1;
  if (cos1<Cut1 && cos2<Cut2){
    if (dMLep<Cut3) it = 0;
    if (dMLep>Cut3) it = 1;
  }
  else if (cos1>Cut1 && cos2>Cut2){
    if (dMLep<Cut3) it = 2;
    if (dMLep>Cut3) it = 3;
  }
  else if ((cos1>Cut1 && cos2<Cut2) || (cos1<Cut1 && cos2>Cut2)){
    if (dMLep<Cut3) it = 4;
    if (dMLep>Cut3) it = 5;
  }
  
  f0e = m_F0[i][j][it];
  f1e = m_F1[i][j][it];
  
  //cout << "it = " << it << " f0e = " << f0e << " f1e = " << f1e << endl;
  
  vector<double> vfe;
  vfe.push_back(f0e);
  vfe.push_back(f1e);
  return vfe;
}

//Make list of events in Data file
void MakeEventListf0f1(int i, int j){
  m_EventList.clear();  

  TFile *fD = TFile::Open("/data/atlas/huonglan/FROMLYON/DATA/resData.root");
  TTree *tD = (TTree*)fD->Get("T"); 
  int nD = tD->GetEntries();
  for (int iev = 0; iev < nD; iev++) {
    tD->GetEntry(iev);
    
    double mt    = tD->GetLeaf("mtophad")->GetValue();
    double HT    = tD->GetLeaf("HT")->GetValue();
    double x     = tD->GetLeaf("cos_tlepCMtpair")->GetValue();
    double y     = tD->GetLeaf("cosWqq")->GetValue();
    double dMLep = tD->GetLeaf("MinLep1")->GetValue();

    int hasBJ = (int)tD->GetLeaf("hasBadJet")->GetValue();
    int RunN  = (int)tD->GetLeaf("RunNumber_ana")->GetValue();
    
    if (RunN >= 180614 && hasBJ==1) continue;

    double cos1 = fabs(x);
    double cos2 = fabs(y);
    
    //sPlots total
    if (mt > Cut4 && HT > Cut5){
      vector<double> fsfb =  vFe(cos1,cos2,dMLep,i,j);
      double f0 = fsfb.at(0);
      double f1 = fsfb.at(1);
      //cout << "f0 = " << f0 << " f1 = " << f1 << endl;
      EventC eventC(1,f1,f0);
      m_EventList.push_back(eventC);
    } 
  }
}

//Fit the number N0 and N1 using the events from data files
std::pair<pair<double,double>,double> Fit(double N0_exp, double N1_exp, int i, int j){
  // Test
  MakeEventListf0f1(i,j);
  cout << "Number of events in the LH " << m_EventList.size() << endl;

  int ifl = 0;
  double step = 0.01;
  
  double fitPar0 = N0_exp;
  double nmax0   = fitPar0 + 4*sqrt(N0_exp);
  double nmin0   = fitPar0 - 4*sqrt(N0_exp);
  
  double fitPar1 = N1_exp;
  double nmax1   = fitPar1 + 4*sqrt(N1_exp);
  double nmin1   =         - 4*sqrt(N1_exp);
  
  double n0best, err0best;
  double n1best, err1best;
  
  TMinuit* myMinuit = new TMinuit(2);
  //myMinuit->Command("SET PRI -1");
  myMinuit->Command("SET NOW");
  myMinuit->Command("SET ERR 0.5");
  myMinuit->Command("SET STR 2");   // try to improve mimimum search
  myMinuit->SetFCN(fcnN0N1);
  myMinuit->mnparm(0,"N0",fitPar0,step,nmin0,nmax0,ifl); 
  myMinuit->mnparm(1,"N1",fitPar1,step,nmin1,nmax1,ifl); 
  //myMinuit->FixParameter(0);
  myMinuit->Command("MIGRAD 500 1");
  myMinuit->GetParameter(0,n0best,err0best); 
  myMinuit->GetParameter(1,n1best,err1best); 
  
  std::cout << "Expected N0 = " << N0_exp << " fitted = " << n0best << " +- " << err0best << std::endl;
  std::cout << "Expected N1 = " << N1_exp << " fitted = " << n1best << " +- " << err1best << std::endl;
  
  double N0 = n0best;
  double N1 = n1best;
  double sig2 = err1best*err1best;

  pair<double,double> N0N1 = make_pair<double,double> (N0,N1);
  
  return make_pair<pair<double,double>,double> (N0N1,sig2);
};


//##############################################################
//####################### MAIN PROGRAM #########################
//##############################################################

void EfficiencyDependOnCut(){
  TTree *t[4];

  TFile *fB = TFile::Open("../MERGEFILES/TESTEFFICIENCY/res_AllBkg.root");
  t[0] = (TTree*)fB->Get("T");

  TFile *fS = TFile::Open("../MERGEFILES/TESTEFFICIENCY/res_Signal.root");
  t[1] = (TTree*)fS->Get("T");

  TFile *fBS = TFile::Open("../MERGEFILES/TESTEFFICIENCY/res_FakeData.root");
  t[2] = (TTree*)fBS->Get("T");

  TFile *fD = TFile::Open("/exp/atlas/huonglan/DATA2011/resData.root");
  t[3] = (TTree*)fD->Get("T");

  double   e1[3][4][31];
  double   e2[3][4][31];   
  double   e3[3][4][31];   
  double   e4[3][4][31];   
  double   e5[3][4][31];   
  double   e6[3][4][31];   
  double iTot[3][4][31];

  for (int k = 0; k < 3; k++){
    for (int i = 0; i < 4; i++){
      for (int j = 0; j < 31; j++){
	iTot[k][i][j] =0 ;
	e1[k][i][j] = 0;
	e2[k][i][j] = 0;
	e3[k][i][j] = 0;
	e4[k][i][j] = 0;
	e5[k][i][j] = 0;
	e6[k][i][j] = 0;
      }
    }
  }

  double mjjj[31];    for (int i = 0; i < 31; i++) { mjjj[i]    = 200 + i*10;}
  double mjjjErr[31]; for (int i = 0; i < 31; i++) { mjjjErr[i] = 0;}

  for (int i = 0; i < 4; i++){
    int nevt = t[i]->GetEntries();
    for (int iev = 0; iev < nevt; iev++) {
      t[i]->GetEntry(iev);

      //cout << "iev " << iev << endl;
      double wei = 1;
      double cos1 = 1, cos2 = 1;
      int hfor = -1;

      double mt   = t[i]->GetLeaf("mtophad")->GetValue();
      double ht   = t[i]->GetLeaf("HT")->GetValue();
      int hasBJ   = (int)t[i]->GetLeaf("hasBadJet")->GetValue();
      if (i==0||i==1||i==2) {
	wei  = t[i]->GetLeaf("wei")->GetValue();
	cos1 = fabs(t[i]->GetLeaf("cos1")->GetValue());
	cos2 = fabs(t[i]->GetLeaf("cos2")->GetValue());
	hfor = (int)t[i]->GetLeaf("hfor")->GetValue();
      } else if (i==3){
	cos1 = fabs(t[i]->GetLeaf("cos_tlepCMtpair")->GetValue());
	cos2 = fabs(t[i]->GetLeaf("cosWqq")->GetValue());
      }
      double dM   = t[i]->GetLeaf("MinLep1")->GetValue();

      if (hasBJ!=0 || hfor==4) continue;

      int it = -1;
      if (ht>620 && mt>200)           it = 0;
      if (ht>500 && ht<620 && mt>200) it = 1;
      if (ht>620 && mt>200 && mt<250) it = 2;

      if (it==-1) continue;
      //cout << "mt " << mt << " ht " << ht << " it " << it << endl;

      for (int j = 0; j < 31; j++){
	double mtCut = mjjj[j];

	//cout << "mtCut " << mtCut << endl;
	  
	if (mt>mtCut){
	  iTot[it][i][j] += wei;
	  
	  if (cos1<Cut1 && cos2<Cut2){
	    if (dM<Cut3) e1[it][i][j] += wei;
	    if (dM>Cut3) e2[it][i][j] += wei;
	  }
	  else if (cos1>Cut1 && cos2>Cut2){
	    if (dM<Cut3) e3[it][i][j] += wei;
	    if (dM>Cut3) e4[it][i][j] += wei;
	  }
	  else if ((cos1>Cut1 && cos2<Cut2) || (cos1<Cut1 && cos2>Cut2)){
	    if (dM<Cut3) e5[it][i][j] += wei;
	    if (dM>Cut3) e6[it][i][j] += wei;
	  }
	}
      }
    }
  }

  double    eB[3][31][6],    eS[3][31][6],    eBS[3][31][6],    eD[3][31][6]; 
  double eBErr[3][31][6], eSErr[3][31][6], eBSErr[3][31][6], eDErr[3][31][6]; 

  for (int i = 0; i < 3; i++){
    for (int j = 0; j < 31; j++){
      //cout << "e1[" << i << "][0][" << j << "] " << e1[i][0][j];
      //cout << " iTot[" << i << "][0][" << j << "] " << iTot[i][0][j] << endl;
      
      eB[i][j][0] = e1[i][0][j]/iTot[i][0][j];
      eB[i][j][1] = e2[i][0][j]/iTot[i][0][j];
      eB[i][j][2] = e3[i][0][j]/iTot[i][0][j];
      eB[i][j][3] = e4[i][0][j]/iTot[i][0][j];
      eB[i][j][4] = e5[i][0][j]/iTot[i][0][j];
      eB[i][j][5] = e6[i][0][j]/iTot[i][0][j];

      if (iTot[i][0][j]==0) { for (int k = 0; k < 6; k++) { eB[i][j][k] = 0; } }

      eS[i][j][0] = e1[i][1][j]/iTot[i][1][j];
      eS[i][j][1] = e2[i][1][j]/iTot[i][1][j];
      eS[i][j][2] = e3[i][1][j]/iTot[i][1][j];
      eS[i][j][3] = e4[i][1][j]/iTot[i][1][j];
      eS[i][j][4] = e5[i][1][j]/iTot[i][1][j];
      eS[i][j][5] = e6[i][1][j]/iTot[i][1][j];

      if (iTot[i][1][j]==0) { for (int k = 0; k < 6; k++) { eS[i][j][k] = 0; } }
      
      eBS[i][j][0] = e1[i][2][j]/iTot[i][2][j];
      eBS[i][j][1] = e2[i][2][j]/iTot[i][2][j];
      eBS[i][j][2] = e3[i][2][j]/iTot[i][2][j];
      eBS[i][j][3] = e4[i][2][j]/iTot[i][2][j];
      eBS[i][j][4] = e5[i][2][j]/iTot[i][2][j];
      eBS[i][j][5] = e6[i][2][j]/iTot[i][2][j];

      if (iTot[i][2][j]==0) { for (int k = 0; k < 6; k++) { eBS[i][j][k] = 0; } }
      
      eD[i][j][0] = e1[i][3][j]/iTot[i][3][j]; 
      eD[i][j][1] = e2[i][3][j]/iTot[i][3][j]; 
      eD[i][j][2] = e3[i][3][j]/iTot[i][3][j]; 
      eD[i][j][3] = e4[i][3][j]/iTot[i][3][j]; 
      eD[i][j][4] = e5[i][3][j]/iTot[i][3][j]; 
      eD[i][j][5] = e6[i][3][j]/iTot[i][3][j]; 

      if (iTot[i][3][j]==0) { for (int k = 0; k < 6; k++) { eD[i][j][k] = 0; } }
      
      //cout << "e1S[" << i << "][" << j << "] = " << e1S[i][j];
      //cout << " iTot[" << i << "][1][" << j << "] = " << iTot[i][1][j] << endl;
    }
  }
  
  //3 is the number of regions
  //31 is the number of the cut for mjjj
  //6 is the number of categories
  for (int i = 0; i < 3; i++){
    for (int j = 0; j < 31; j++){
      for (int k = 0; k < 6; k++){
	eBErr[i][j][k]  = sqrt(eB[i][j][k]*(1-eB[i][j][k])/iTot[i][0][j]);
	eSErr[i][j][k]  = sqrt(eS[i][j][k]*(1-eS[i][j][k])/iTot[i][1][j]);
	eBSErr[i][j][k] = sqrt(eBS[i][j][k]*(1-eBS[i][j][k])/iTot[i][2][j]);
	eDErr[i][j][k]  = sqrt(eD[i][j][k]*(1-eD[i][j][k])/iTot[i][3][j]);

	m_F1[i][j][k] = eS[i][j][k];
	m_F0[i][j][k] = eB[i][j][k];
	m_N1exp[i][j] = iTot[i][1][j];
	m_N0exp[i][j] = iTot[i][0][j];

	cout << "region " << i << " cut mjjj " << mjjj[j] << " cate " << k+1 ;
	cout << " iTotB = " << iTot[i][0][j] ;
	cout << " iTotS = " << iTot[i][1][j] << endl;
      }
    }
  }

  //3 is the number of regions
  //31 is the number of the cut for mjjj
  double N0fit[3][31], N1fit[3][31], N1err[3][31];
  for (int i = 0; i < 3; i++){
    for (int j = 0; j < 31; j++){
      pair<pair<double,double>,double> N0N1sig2 = Fit(m_N0exp[i][j],m_N1exp[i][j],i,j);
      pair<double,double> N0N1 = N0N1sig2.first;
      double N0   = N0N1.first; 
      double N1   = N0N1.second; 
      double sig2 = N0N1sig2.second;
      N0fit[i][j] = N0;
      N1fit[i][j] = N1;
      N1err[i][j] = sig2;
    }
  }

  TH2F *frame1 =  new TH2F("Frame1", "", 31, 200, 500, 1000, 0.001, 1);
  frame1->SetStats(0);
  TH2F *frame2 =  new TH2F("Frame2", "", 31, 200, 500, 1000, 0.001, 1);
  frame2->SetStats(0);
  TH2F *frame3 =  new TH2F("Frame3", "", 31, 200, 500, 1000, 0.001, 1);
  frame3->SetStats(0);

  TLegend *legend = new TLegend(0.3,0.6,0.7,0.75);
  legend->SetBorderSize(0);
  legend->SetFillColor(0);

  TString name = "";
  TString region[3] = {"region1","region2","region3"};
  TString cat[6] = {"cat1","cat2","cat3","cat4","cat5","cat6"};

//   TGraphErrors *heffB[3][6];
//   TGraphErrors *heffS[3][6];
//   TGraphErrors *heffBS[3][6];
//   TGraphErrors *heffD[3][6];

  TGraph *heffB[3][6];
  TGraph *heffS[3][6];
  TGraph *heffBS[3][6];
  TGraph *heffD[3][6];

  //*************Define histograms on N1************
  TGraphErrors *hN1[3];
  for (int i = 0; i < 3; i++){
    double N1fittemp[31], N1errtemp[31];
    for (int j = 0; j < 31; j++){ 
      N1fittemp[j] = N1fit[i][j];
      N1errtemp[j] = N1err[i][j]; 
      cout << "mjjj = " << mjjj[j] << endl;
      cout << " m_N1exp[" << i << "][" << j << "]= " << m_N1exp[i][j] ;
      cout << " m_N0exp[" << i << "][" << j << "]= " << m_N0exp[i][j] ;
      cout << " N1fit[" << i << "][" << j << "]= " << N1fit[i][j] ;
      cout << " N0fit[" << i << "][" << j << "]= " << N0fit[i][j] ;
      cout << " N1err[" << i << "][" << j << "]= " << N1err[i][j] << endl;
    }
    hN1[i] = new TGraphErrors(31,mjjj,N1fittemp,mjjjErr,N1errtemp);
  }

  //*************Define histograms on efficiencies****************
  for (int i = 0; i < 3; i++){
    for (int j = 0; j < 6; j++){
      double eff[31]; 
      double effErr[31];

      name = "heffB"; name += region[i]; name += cat[j];
      for (int ic = 0; ic < 31; ic++) { 
	eff[ic]    = eB[i][ic][j];
	effErr[ic] = eBErr[i][ic][j];
      }
      //heffB[i][j] = new TGraphErrors(31, mjjj, eff, mjjjErr, effErr);
      heffB[i][j] = new TGraph(31, mjjj, eff);
      heffB[i][j]->SetLineColor(8);
      heffB[i][j]->SetLineWidth(2);
      
      name = "heffS"; name += region[i]; name += cat[j];
      for (int ic = 0; ic < 31; ic++) { 
	eff[ic]    = eS[i][ic][j];
	effErr[ic] = eSErr[i][ic][j];
      }
      //heffS[i][j] = new TGraphErrors(31, mjjj, eff, mjjjErr, effErr);
      heffS[i][j] = new TGraph(31, mjjj, eff);
      heffS[i][j]->SetLineColor(2);
      heffS[i][j]->SetLineWidth(2);
      
      name = "heffBS"; name += region[i]; name += cat[j];
      for (int ic = 0; ic < 31; ic++) { 
	eff[ic]    = eBS[i][ic][j];
	effErr[ic] = eBSErr[i][ic][j];
      }
      //heffBS[i][j] = new TGraphErrors(31, mjjj, eff, mjjjErr, effErr);
      heffBS[i][j] = new TGraph(31, mjjj, eff);
      heffBS[i][j]->SetLineColor(kMagenta);
      heffBS[i][j]->SetLineWidth(2);
      
      name = "heffD"; name += region[i]; name += cat[j];
      for (int ic = 0; ic < 31; ic++) { 
	eff[ic]    = eD[i][ic][j];
	effErr[ic] = eDErr[i][ic][j];
      }
      //heffD[i][j] = new TGraphErrors(31, mjjj, eff, mjjjErr, effErr);
      heffD[i][j] = new TGraph(31, mjjj, eff);
      heffD[i][j]->SetLineWidth(2);
    }
  }

  legend->AddEntry(heffB[0][1],"BkgMC");
  legend->AddEntry(heffS[0][1],"SigMC");
  legend->AddEntry(heffBS[0][1],"Sig+BkgMC");
  legend->AddEntry(heffD[0][1],"Data");

  //****************** DRAWING *********************

  TCanvas *cN1 = new TCanvas("cN1", "cN1", 1000, 400);
  cN1->Divide(3);
  
  cN1->cd(1);
  hN1[0]->Draw("ALP");
  cN1->cd(2);
  hN1[1]->Draw("ALP");
  cN1->cd(3);
  hN1[2]->Draw("ALP");

  cN1->Print("N1fit.eps");

  TCanvas *c[6];
  for (int i = 0; i < 6; i++){
    name = "Cat"; name += (i+1);
    c[i] = new TCanvas(name, name, 1000, 400);
    c[i]->Divide(3);

    c[i]->cd(1);
    name += ": mt>250 && HT>620";
    frame1->SetTitle(name);
    frame1->Draw();
    heffS[0][i]->Draw("LP");
    heffD[0][i]->Draw();
    heffBS[0][i]->Draw();
    heffB[0][i]->Draw();
    legend->Draw("same");

    c[i]->cd(2);
    name = "mt>200 && HT>500 && HT<620";
    frame2->SetTitle(name);
    frame2->Draw();
    heffS[1][i]->Draw("LP");
    heffD[1][i]->Draw();
    heffBS[1][i]->Draw();
    heffB[1][i]->Draw();
    legend->Draw("same");

    c[i]->cd(3);
    name = "mt>200 && mt<250 && HT>620";
    frame3->SetTitle(name);
    frame3->Draw();
    heffS[2][i]->Draw("LP");
    heffD[2][i]->Draw();
    heffBS[2][i]->Draw();
    heffB[2][i]->Draw();
    legend->Draw("same");

    name = "Cat"; name += (i+1); name += ".eps";
    c[i]->Print(name);
  }

}
