#include <iostream>
#include <TFile.h>
#include <TTree.h>
#include <TH1F.h>
#include <TH2F.h>
#include <TString.h>
#include <TLeaf.h>
#include <TRandom.h>
#include <TMinuit.h>
#include <TF1.h>
#include <TCanvas.h>
#include <vector>

#include <iostream>

using namespace std;

void sPlots_NewRel_N0known();

//Set the cut values
double Cut1 = 0.7;
double Cut2 = 0.7;
double Cut3 = 48;
double Cut4 = 250;
double Cut5 = 620;

//global variables 
int    m_icount = 0;
double m_iTot[2];              //for (int i = 0; i < 2; i++){ m_iTot[i] = 0;}
double m_iCatS[6], m_iCatB[6]; //for (int i = 0; i < 6; i++){ m_iCatS[i] = 0; m_iCatB[i] = 0;}
double m_F1[6];                //for (int i = 0; i < 6; i++){ m_F1[i] = 0; }
double m_F0[6];                //for (int i = 0; i < 6; i++){ m_F0[i] = 0; }
double m_N1 = 0., m_N0 = 0., m_sig2 = 0.;

TH1F *h[7];

//Define class Event
class EventC {
public:
  EventC() { m_w = 1; m_fs = 1; m_fb = 0; m_cat = 1; }
  EventC(double w0, double fs0, double fb0) { m_w = w0; m_fs = fs0; m_fb = fb0; }
  EventC(double w0, int cat0) { m_w = w0; m_cat = cat0; m_fs = 1; m_fb = 0; }
  double w() { return m_w; }
  double fs() { return m_fs; }
  double fb() { return m_fb; }
  int cat() { return m_cat; }
  void fs(double fs0) { m_fs = fs0; }
  void fb(double fb0) { m_fb = fb0; }

private:
  double m_w, m_fs, m_fb;
  int m_cat;
};

std::vector<EventC> m_EventList;
bool m_ListAlreadyDone = false;

void fcnN1(Int_t &npar, Double_t *gin, Double_t &f, Double_t *par, Int_t iflag) {
  double L = 0;
  for (unsigned int iev = 0; iev < m_EventList.size(); iev++) {
    EventC evt = m_EventList[iev];
    double smt = m_N0*evt.fb() + par[0]*evt.fs();
    if (smt > 0) L += evt.w()*log(smt);
  }
  f = par[0] - L; 
};

void fcnN0N1(Int_t &npar, Double_t *gin, Double_t &f, Double_t *par, Int_t iflag) {
  double L = 0;
  for (unsigned int iev = 0; iev < m_EventList.size(); iev++) {
    EventC evt = m_EventList[iev];
    double smt = par[0]*evt.fb() + par[1]*evt.fs();
    if (smt > 0) L += evt.w()*log(smt);
  }
  f = par[0] + par[1] - L; 
};

vector<double> InvMatrix(double a, double b, double c, double d){
  double detM = a*d - b*c;
  double k = 1./detM;
  double V00 = k*d;
  double V01 = k*(-b);
  double V10 = k*(-c);
  double V11 = k*a;
  
  vector<double> Vinv;
  Vinv.push_back(V00);
  Vinv.push_back(V01);
  Vinv.push_back(V10);
  Vinv.push_back(V11);

  return Vinv;
}

void CalF0F1(){
  //######## COMPUTE the sWeights
  TString filename = "";
  TString type[2] = {"AllBkgTopMCAtNLO","Signal115420"};
  TFile *file[2];
  TTree *t[2];
  
  for (int i = 0; i < 2; i++){
    filename = "./MERGEFILES/res_"; filename += type[i]; filename += "v3.root";
    file[i] = TFile::Open(filename);
    t[i]    = (TTree*)file[i]->Get("T");
  }

  for (int i = 0; i < 2; i++){
    int nevt = t[i]->GetEntries();
    for (int iev = 0; iev < nevt; iev++) {
      t[i]->GetEntry(iev);
	    
      double mt     = t[i]->GetLeaf("mtophad")->GetValue();
      double HT     = t[i]->GetLeaf("HT")->GetValue();
      double x      = t[i]->GetLeaf("cos1")->GetValue();
      double y      = t[i]->GetLeaf("cos2")->GetValue();
      double dMLep  = t[i]->GetLeaf("MinLep1")->GetValue();
      double WeiTot = t[i]->GetLeaf("wei")->GetValue();

      //cout << "WeiTot = " << WeiTot << endl;

      int hasBJ = (int)t[i]->GetLeaf("hasBadJet")->GetValue();
      int hf    = (int)t[i]->GetLeaf("hfor")->GetValue();

      if (hf!=4 && hasBJ==0){
	
	double cos1 = fabs(x);
	double cos2 = fabs(y);
      
	if (mt > Cut4 && HT > Cut5) {
	  m_iTot[i] += WeiTot;
	  int it = -1;
	  if (cos1<Cut1 && cos2<Cut2){
	    if (dMLep<Cut3) it = 0;
	    if (dMLep>Cut3) it = 1;
	  }
	  else if (cos1>Cut1 && cos2>Cut2){
	    if (dMLep<Cut3) it = 2;
	    if (dMLep>Cut3) it = 3;
	  }
	  else if ((cos1>Cut1 && cos2<Cut2) || (cos1<Cut1 && cos2>Cut2)){
	    if (dMLep<Cut3) it = 4;
	    if (dMLep>Cut3) it = 5;
	  } else {
	    cout << "Unknown category !!" << endl;
	  }
	  if (i == 0) 
	    m_iCatB[it] += WeiTot;
	  else
	    m_iCatS[it] += WeiTot;
	}
      }
    }//Read each file
  }//End of reading 2 files

  m_N0 = m_iTot[0];
  m_N1 = m_iTot[1];

  cout << "To calculate efficiencies using " << type[1] << endl;
  cout << "iTot0 = " << m_iTot[0] << endl;
  cout << "iTot1 = " << m_iTot[1] << endl;

  double sigmaInv = 0;
  double sumeffS = 0, sumeffB = 0;
  for (int i = 0; i < 6; i++){
    m_F1[i] = m_iCatS[i]/m_iTot[1];
    m_F0[i] = m_iCatB[i]/m_iTot[0];
    sumeffS += m_F1[i];
    sumeffB += m_F0[i];
    cout << "m_F1[" << i << "] = " << m_F1[i] << endl;
    cout << "m_F0[" << i << "] = " << m_F0[i] << endl;

    double term = m_F1[i]*m_F1[i]/(m_N1*m_F1[i] + m_N0*m_F0[i]);
    if (m_F1[i]==0 && m_F0[i]==0)  term = 0;
    sigmaInv += term;
  }
}

vector<double> vFe(double cos1,double cos2,double dMLep)
{
  double f0e = 0;
  double f1e = 0;
  
  //cout << "cos1 = " << cos1 << " cos2 = " << cos2 << " dMLep = " << dMLep << endl;
  
  int it = -1;
  if (cos1<Cut1 && cos2<Cut2){
    if (dMLep<Cut3) it = 0;
    if (dMLep>Cut3) it = 1;
  }
  else if (cos1>Cut1 && cos2>Cut2){
    if (dMLep<Cut3) it = 2;
    if (dMLep>Cut3) it = 3;
  }
  else if ((cos1>Cut1 && cos2<Cut2) || (cos1<Cut1 && cos2>Cut2)){
    if (dMLep<Cut3) it = 4;
    if (dMLep>Cut3) it = 5;
  }
  
  f0e = m_F0[it];
  f1e = m_F1[it];
  
  //cout << "it = " << it << " f0e = " << f0e << " f1e = " << f1e << endl;
  
  vector<double> vfe;
  vfe.push_back(f0e);
  vfe.push_back(f1e);
  return vfe;
}

void MakeEventListf0f1(){
  m_ListAlreadyDone = true;
  //######## COMPUTE the sWeights
  TString filename = "";
  TString type[2] = {"AllBkgTopMCAtNLO","Signal115426"};
  TFile *file[2];
  TTree *t[2];
  
  for (int i = 0; i < 2; i++){
    filename = "./MERGEFILES/res_"; filename += type[i]; filename += "v3.root";
    file[i] = TFile::Open(filename);
    t[i]    = (TTree*)file[i]->Get("T");
  }
  
  for (int i = 0; i < 2; i++){
    int nevt = t[i]->GetEntries();
    for (int iev = 0; iev < nevt; iev++) {
      t[i]->GetEntry(iev);
      
      double mt     = t[i]->GetLeaf("mtophad")->GetValue();
      double HT     = t[i]->GetLeaf("HT")->GetValue();
      double x      = t[i]->GetLeaf("cos1")->GetValue();
      double y      = t[i]->GetLeaf("cos2")->GetValue();
      double dMLep  = t[i]->GetLeaf("MinLep1")->GetValue();
      double WeiTot = t[i]->GetLeaf("wei")->GetValue();

      // Yes some events have 0 weight due to PU reweighting
      // (just to save time, remove them)
      if (fabs(WeiTot) < 1e-9) continue; 
      //cout << "Reading event " << iev << " of type " << i << " w = " << WeiTot << endl; 
      
      //cout << "WeiTot = " << WeiTot << endl;
      
      int hasBJ  = (int)t[i]->GetLeaf("hasBadJet")->GetValue();
      int hf     = (int)t[i]->GetLeaf("hfor")->GetValue();

      if (hf!=4 && hasBJ==0){
	
	double cos1 = fabs(x);
	double cos2 = fabs(y);
	
	if (mt > Cut4 && HT > Cut5) {
	  vector<double> fsfb =  vFe(cos1,cos2,dMLep);
	  double f0 = fsfb.at(0);
	  double f1 = fsfb.at(1);
	  //cout << "f0 = " << f0 << " f1 = " << f1 << endl;
	  EventC eventC(WeiTot,f1,f0);
	  m_EventList.push_back(eventC);
	}
      }//Read each file
    }//End of reading 2 files
  }
}

void Fit(bool FitN0){
  // Test
  if (!m_ListAlreadyDone) MakeEventListf0f1();
  cout << "Number of events in the LH " << m_EventList.size() << endl;

  if (FitN0){

    double N0_exp = m_iTot[0];
    double N1_exp = m_iTot[1];

    cout << "FitN0N1 " << " N0_exp " << N0_exp << " N1_exp " << N1_exp << endl;

    int ifl = 0;
    double step = 0.01;

    double fitPar0 = N0_exp;
    double nmax0   = fitPar0 + 4*sqrt(N0_exp);
    double nmin0   = fitPar0 - 4*sqrt(N0_exp);

    double fitPar1 = N1_exp;
    double nmax1   = fitPar1 + 4*sqrt(N1_exp);
    double nmin1   =         - 4*sqrt(N1_exp);

    double n0best, err0best;
    double n1best, err1best;

    TMinuit* myMinuit = new TMinuit(2);
    //myMinuit->Command("SET PRI -1");
    myMinuit->Command("SET NOW");
    myMinuit->Command("SET ERR 0.5");
    myMinuit->Command("SET STR 2");   // try to improve mimimum search
    myMinuit->SetFCN(fcnN0N1);
    myMinuit->mnparm(0,"N0",fitPar0,step,nmin0,nmax0,ifl); 
    myMinuit->mnparm(1,"N1",fitPar1,step,nmin1,nmax1,ifl); 
    //myMinuit->FixParameter(0);
    myMinuit->Command("MIGRAD 500 1");
    myMinuit->GetParameter(0,n0best,err0best); 
    myMinuit->GetParameter(1,n1best,err1best); 

    std::cout << "Expected N0 = " << N0_exp << " fitted = " << n0best << " +- " << err0best << std::endl;
    std::cout << "Expected N1 = " << N1_exp << " fitted = " << n1best << " +- " << err1best << std::endl;
    
    m_N0 = n0best;
    m_N1 = n1best;
  }
  else {
    double N0_exp = m_iTot[0];
    double N1_exp = m_iTot[1];

    cout << "FitN1 " << " N0_exp " << N0_exp << " N1_exp " << N1_exp << endl;

    int ifl = 0;
    double step = 0.01;
    double fitPar1 = N1_exp;
    double nmax1   = fitPar1 + 4*sqrt(N1_exp);
    double nmin1   = - 4*sqrt(N1_exp);
    double n1best, err1best;
    TMinuit* myMinuitN1 = new TMinuit(1);
    //myMinuitN1->Command("SET PRI -1");
    myMinuitN1->Command("SET NOW");
    myMinuitN1->Command("SET ERR 0.5");
    myMinuitN1->Command("SET STR 2");   // try to improve mimimum search
    myMinuitN1->SetFCN(fcnN1);
    myMinuitN1->mnparm(0,"N1",fitPar1,step,nmin1,nmax1,ifl); 
    //myMinuitN1->FixParameter(0);
    myMinuitN1->Command("MIGRAD 500 1");
    myMinuitN1->GetParameter(0,n1best,err1best); 
    std::cout << "Expected N0 = " << N0_exp << endl;
    std::cout << "Expected N1 = " << N1_exp << " fitted = " << n1best << " +- " << err1best << std::endl;
   
    m_N1 = n1best;
    m_sig2 = err1best*err1best;
  }

  cout << "bool FitN0 " << FitN0 << " m_N0 = " << m_N0 << " m_N1 = " << m_N1 << endl;
  //cout << "m_N0 = " << m_N0 << " m_N1 = " << m_N1 << endl;
}

//########################################################################
//############################ MAIN PROGRAM ##############################
//########################################################################

void sPlots_NewRel(){
  cout << "Cut1 = " << Cut1 << endl;
  cout << "Cut2 = " << Cut2 << endl;
  cout << "Cut3 = " << Cut3 << endl;
  cout << "Cut4 = " << Cut4 << endl;
  cout << "Cut5 = " << Cut5 << endl;

  //Compute the efficiencies first
  CalF0F1();

  //Reset m_iTot[0] and m_iTot[1]
  m_iTot[0] = 0.; m_iTot[1] = 0.;
  //double F0N[6]

  //Now compute m_N0, m_N1 
  TString filename = "";
  TString type[2] = {"AllBkgTopMCAtNLO","Signal115426"};
  TFile *file[2];
  TTree *t[2];
  
  for (int i = 0; i < 2; i++){
    filename = "./MERGEFILES/res_"; filename += type[i]; filename += "v3.root";
    file[i] = TFile::Open(filename);
    t[i]    = (TTree*)file[i]->Get("T");
  }

  for (int i = 0; i < 2; i++){
    int nevt = t[i]->GetEntries();
    for (int iev = 0; iev < nevt; iev++) {
      t[i]->GetEntry(iev);
	    
      double mt     = t[i]->GetLeaf("mtophad")->GetValue();
      double HT     = t[i]->GetLeaf("HT")->GetValue();
      double WeiTot = t[i]->GetLeaf("wei")->GetValue();

      int hasBJ = (int)t[i]->GetLeaf("hasBadJet")->GetValue();
      int    hf = (int)t[i]->GetLeaf("hfor")->GetValue();

      if (hf!=4 && hasBJ==0){
	if (mt > Cut4 && HT > Cut5) {
	  m_icount++;
	  m_iTot[i] += WeiTot;
	}
      }
    }//Read each file
  }//End of reading 2 files

  cout << "In main macro" << endl;
  cout << "iTot0 = " << m_iTot[0] << endl;
  cout << "iTot1 = " << m_iTot[1] << endl;

  m_N1 = m_iTot[1];
  m_N0 = m_iTot[0];

  cout << "m_icount = " << m_icount << endl;

  //sPlots N0 known
  cout << "***********************************************" << endl;
  cout << "************** sPlot N0 known *****************" << endl;
  cout << "***********************************************" << endl;
  sPlots_NewRel_N0known();

  cout << "***********************************************" << endl;
  cout << "************* sPlot N0 unknown ****************" << endl;
  cout << "***********************************************" << endl;

  //Get right N0 and N1 for standard sPlot
  Fit(true);
 
  double VI00 = 0, VI01 = 0, VI10 = 0, VI11 = 0;
  //start reading 2 files
  //int m_icount = 0;
  for (int i = 0; i < 2; i++){
    int nevt = t[i]->GetEntries();
    for (int iev = 0; iev < nevt; iev++) {
      t[i]->GetEntry(iev);
	    
      double mt     = t[i]->GetLeaf("mtophad")->GetValue();
      double HT     = t[i]->GetLeaf("HT")->GetValue();
      double x      = t[i]->GetLeaf("cos1")->GetValue();
      double y      = t[i]->GetLeaf("cos2")->GetValue();
      double dMLep  = t[i]->GetLeaf("MinLep1")->GetValue();
      double WeiTot = t[i]->GetLeaf("wei")->GetValue();

      double cos1 = fabs(x);
      double cos2 = fabs(y);
      
      int hasBJ  = (int)t[i]->GetLeaf("hasBadJet")->GetValue();
      int     hf = (int)t[i]->GetLeaf("hfor")->GetValue();

      if (hf!=4 && hasBJ==0){
	if (mt > Cut4 && HT > Cut5){
	  //Compute the matrix
	  vector<double> veff = vFe(cos1,cos2,dMLep);
	  double F0e = veff.at(0);
	  double F1e = veff.at(1);
	  
	  //cout << "F0e = " << F0e << " F1e = " << F1e << endl;
	  
	  double vi00;
	  double vi01;
	  double vi10;
	  double vi11;
	  
	  vi00 = WeiTot*F0e*F0e/(m_N1*F1e + m_N0*F0e)/(m_N1*F1e + m_N0*F0e);
	  vi01 = WeiTot*F0e*F1e/(m_N1*F1e + m_N0*F0e)/(m_N1*F1e + m_N0*F0e);
	  vi10 = WeiTot*F1e*F0e/(m_N1*F1e + m_N0*F0e)/(m_N1*F1e + m_N0*F0e);
	  vi11 = WeiTot*F1e*F1e/(m_N1*F1e + m_N0*F0e)/(m_N1*F1e + m_N0*F0e);

	  VI00 += vi00;
	  VI01 += vi01;
	  VI10 += vi10;
	  VI11 += vi11;
	}
      }
    }//Read each file
  }//End of reading 2 files

  cout << "Main macro : m_N0 = " << m_N0 << " m_N1 = " << m_N1 << endl;

  cout << "VI00 = " << VI00 << endl;
  cout << "VI01 = " << VI01 << endl;
  cout << "VI10 = " << VI10 << endl;
  cout << "VI11 = " << VI11 << endl;
  vector<double> Vinv = InvMatrix(VI00,VI01,VI10,VI11);
  double V00 = Vinv.at(0);
  double V01 = Vinv.at(1);
  double V10 = Vinv.at(2);
  double V11 = Vinv.at(3);
  cout << "V00 = " << V00 << endl;
  cout << "V01 = " << V01 << endl;
  cout << "V10 = " << V10 << endl;
  cout << "V11 = " << V11 << endl;
  double sigmaN0 = sqrt(V00);
  double sigmaN1 = sqrt(V11);
  double rho     = V10/sqrt(V00*V11);
  cout << "sigmaN0 = " << sigmaN0 << " sigmaN1 = " << sigmaN1 << " rho = " << rho << endl; 

  TH1F *hSigN  = new TH1F("hSigN",     "", 150, 0, 1500);
  TH1F *hBkgN  = new TH1F("hBkgN",     "", 150, 0, 1500);
  TH2F *h2DSig = new TH2F("h2DSig",    "", 150, 0, 1500, 50, 500, 1500);
  TH2F *h2DBkg = new TH2F("h2DBkg",    "", 150, 0, 1500, 50, 500, 1500);

  //Try standard sPlots
  double SumsP1 = 0.;
  double SumsP0 = 0.;
  double *sPN = new double[m_icount];
  double *mtN = new double[m_icount];
  int icc = 0;
  for (int i = 0; i < 2; i++){
    int nevt = t[i]->GetEntries();
    for (int iev = 0; iev < nevt; iev++) {
      t[i]->GetEntry(iev);
      
      double mt    = t[i]->GetLeaf("mtophad")->GetValue();
      double HT    = t[i]->GetLeaf("HT")->GetValue();
      double x     = t[i]->GetLeaf("cos1")->GetValue();
      double y     = t[i]->GetLeaf("cos2")->GetValue();
      double dMLep = t[i]->GetLeaf("MinLep1")->GetValue();
      double WeiTot = t[i]->GetLeaf("wei")->GetValue();
      if (fabs(WeiTot) < 1e-9) continue; 
      
      int hasBJ  = (int)t[i]->GetLeaf("hasBadJet")->GetValue();
      int     hf = (int)t[i]->GetLeaf("hfor")->GetValue();

      if (hf!=4 && hasBJ==0){
	
	double cos1 = fabs(x);
	double cos2 = fabs(y);

	//sPlots total
	if (mt > Cut4 && HT > Cut5){
	  vector<double> veff = vFe(cos1,cos2,dMLep);
	  double F0e = veff.at(0);
	  double F1e = veff.at(1);
	  
	  double sP1 = (V11*F1e + V10*F0e)/(m_N1*F1e + m_N0*F0e);
	  double sP0 = (V00*F0e + V10*F1e)/(m_N1*F1e + m_N0*F0e);
	  
	  hSigN->Fill(mt,sP1*WeiTot);
	  hBkgN->Fill(mt,sP0*WeiTot);
	  h2DSig->Fill(mt,HT,sP1*WeiTot);
	  h2DBkg->Fill(mt,HT,sP0*WeiTot);
	  mtN[icc] = mt;
	  sPN[icc] = sP1*sP1*WeiTot;
	  icc++;
	  
	  SumsP1 += sP1*WeiTot;
	  SumsP0 += sP0*WeiTot;
	}
      }
    }//Read each file
  }//End of reading 2 files
  
  cout << "SumsP0 = " << SumsP0 << " SumsP1 = " << SumsP1 << endl;

  cout << endl << "*******************" << endl;
  cout << "STANDARD SPLOTS" << endl;
  cout << "hSigN integral  " << hSigN->Integral(0,1500)  << endl;
  cout << "hBkgN integral  " << hBkgN->Integral(0,1500)  << endl;
  
  for (int i = 1; i < 151; i++){
    double vb1 = (i-1)*10;
    double vb2 = i*10;
    double ErrBin = 0.;
    for (int j = 0; j < m_icount; j++){
      double mt = mtN[j];
      double sp = sPN[j];
      if (mt >= vb1 && mt < vb2){
	ErrBin += sp;
      }
    }
    if (ErrBin<0) cout << "ErrBin is negative in sPlot standard " << i << " " << ErrBin << endl;
    hSigN->SetBinError(i,sqrt(fabs(ErrBin)));
  }

  //TH2F *frame = new TH2F("frame", "", 35, 250, 600, 50, -10, 40);

  TCanvas *c = new TCanvas("c", "cStandard", 900, 400);
  c->Divide(2);
  c->cd(1); 
  //frame->Draw();
  double x1 = 250;
  double x2 = 600 - 0.01;
  TAxis *ax = hSigN->GetXaxis();
  int b1 = ax->FindBin(x1);
  int b2 = ax->FindBin(x2);
  ax->SetRange(b1,b2);
  hSigN->SetLineColor(2);
  hSigN->SetLineWidth(1);
  hSigN->DrawCopy("E1");
  hSigN->SetLineColor(1);
  hSigN->SetLineWidth(2);
  hSigN->DrawCopy("hist,same");
  c->cd(2);
  TAxis *axB = hBkgN->GetXaxis();
  int b1B = axB->FindBin(x1);
  int b2B = axB->FindBin(x2);
  axB->SetRange(b1B,b2B);
  hBkgN->SetLineColor(4);
  hBkgN->SetLineWidth(2);
  hBkgN->Draw();

  TCanvas *c2D = new TCanvas("c2D", "", 900, 400);
  c2D->Divide(2);
  c2D->cd(1);
  h2DSig->Draw("colz");
  c2D->cd(2);
  h2DBkg->Draw("colz");
}


//##################################################################
//########### FUNCTION FOR BACKGROUND YIELD KNOWN ##################
//##################################################################

void sPlots_NewRel_N0known(){

  Fit(false);

  cout << "in function N0 known " << m_icount << endl;
  cout << "m_sig2 = " << m_sig2 << endl;
  cout << "m_N0 = " << m_N0 << " m_N1 = " << m_N1 << endl;

  double k = (m_sig2 - m_N1)/(m_N0 - (m_sig2 - m_N1));
  
  TH2F *twoDim_mt_HT_Sig     = new TH2F("mt_HT_Sig",     "", 75, 250, 1500, 50, 500, 2000); 
  TH2F *twoDim_mt_HT_Sig_esP = new TH2F("mt_HT_Sig_esP", "", 75, 250, 1500, 50, 500, 2000); 
  
  h[0]  = new TH1F("All1fb",       "",  150,  0,  1500);
  h[1]  = new TH1F("Sig1fb",       "",  150,  0,  1500);
  h[2]  = new TH1F("Bkg1fb",       "",  150,  0,  1500);
  h[3]  = new TH1F("sPlotAll",     "",  150,  0,  1500);
  h[4]  = new TH1F("sPlotSig",     "",  150,  0,  1500);
  h[5]  = new TH1F("sPlotBkg",     "",  150,  0,  1500);
  h[6]  = new TH1F("esPlotSig",    "",  150,  0,  1500);
  
  double *mtV  = new double[m_icount];
  double *err  = new double[m_icount];
  double *errN = new double[m_icount];
  int icc = 0;
  
  TString filename = "";
  TString type[2] = {"AllBkgTopMCAtNLO","Signal115426"};
  TFile *file[2];
  TTree *t[2];
  for (int i = 0; i < 2; i++){
    filename = "./MERGEFILES/res_"; filename += type[i]; filename += "v3.root";
    file[i] = TFile::Open(filename);
    t[i]    = (TTree*)file[i]->Get("T");
  }

  for (int i = 0; i < 2; i++){
    int nevt = t[i]->GetEntries();
    for (int iev = 0; iev < nevt; iev++) {
      t[i]->GetEntry(iev);
      
      double mt     = t[i]->GetLeaf("mtophad")->GetValue();
      double HT     = t[i]->GetLeaf("HT")->GetValue();
      double x      = t[i]->GetLeaf("cos1")->GetValue();
      double y      = t[i]->GetLeaf("cos2")->GetValue();
      double dMLep  = t[i]->GetLeaf("MinLep1")->GetValue();
      double WeiTot = t[i]->GetLeaf("wei")->GetValue();

      int hasBJ  = (int)t[i]->GetLeaf("hasBadJet")->GetValue();
      int     hf = (int)t[i]->GetLeaf("hfor")->GetValue();
      
      if (hf!=4 && hasBJ==0){
	
	double cos1 = fabs(x);
	double cos2 = fabs(y);
	
	//sPlots total
	if (mt > Cut4 && HT > Cut5){
	  vector<double> veff = vFe(cos1,cos2,dMLep);
	  double F0e = veff.at(0);
	  double F1e = veff.at(1);
	  
	  double sWei  = m_sig2*F1e*WeiTot/(m_N1*F1e + m_N0*F0e);
	  double sWeiN = sWei*(k+1) - k*WeiTot;

	  //cout << "sWei  = " << sWei << endl;
	  //cout << "sWeiN = " << sWeiN << endl;
	  
	  //For 1 fb-1
	  h[0]->Fill(mt,WeiTot);
	  h[3]->Fill(mt,sWei);//sPlot
	  h[6]->Fill(mt,sWeiN);//esPlot

	  if (i==0) {
	    h[2]->Fill(mt,WeiTot);
	    h[5]->Fill(mt,sWei);
	  }
	  if (i==1){
	    h[1]->Fill(mt,WeiTot);
	    //h[4]->Fill(mt,sWei);
	    twoDim_mt_HT_Sig->Fill(mt,HT,sWei);
	    twoDim_mt_HT_Sig_esP->Fill(mt,HT,sWeiN);	
	  }
	  
	  //uncertainty
	  mtV[icc] = mt;
	  if (WeiTot==0){
	    err[icc] = 0;
	    errN[icc] = 0;
	  } else {
	    err[icc] = sWei*sWei/WeiTot;
	    errN[icc] = sWeiN*sWeiN/WeiTot;
	  }
	  icc++;
	}
      }
    }//Read each file
  }//End of reading 2 files
  
  h[2]->Scale(1./h[2]->Integral(0,1500));

  for (int i = 1; i < 151; i++){
    double sP = h[3]->GetBinContent(i);
    double sB = h[5]->GetBinContent(i);
    double sW  = sP - sB;

    //double B  = (m_sig2 - m_N1)*h[2]->GetBinContent(i);
    //double W  = sP - B;
    //double Tot1fb = h[0]->GetBinContent(i);
    //double BkgPure = Tot1fb - sP;
    
    double vb1 = 10*(i-1);
    double vb2 = 10*i;
    double SumErrBin = 0;
    double SumErrNBin = 0;
    for (int ic = 0; ic < m_icount; ic++){
      double mt  = mtV[ic];
      double errV  = err[ic];
      double errNV = errN[ic];
      if (mt>=vb1 && mt<vb2){
	SumErrBin += errV;
	SumErrNBin += errNV;
      }
    }
    h[4]->SetBinContent(i,sW);
    if (SumErrBin < 0) cout << "Warning negative error squared in bin " << i << " for sP" << endl; 
    h[4]->SetBinError(i,sqrt(fabs(SumErrBin)));
    if (SumErrNBin < 0) cout << "Warning negative error squared in bin " << i << " for esP" << endl; 
    h[6]->SetBinError(i,sqrt(fabs(SumErrNBin)));
  }
  
  cout << "h2 integral is " << h[2]->Integral(0,1500) << endl;
  cout << "h4 integral is " << h[4]->Integral(0,1000) << endl;
  cout << "h6 integral is " << h[6]->Integral(0,1500) << endl;

  double x1 = 250;
  double x2 = 600 - 0.01;
  for (int i = 0; i < 7; i++) {
    //h[i]->Rebin(2);
    h[i]->SetLineWidth(2);
    TAxis *ax = h[i]->GetXaxis();
    int b1 = ax->FindBin(x1);
    int b2 = ax->FindBin(x2);
    ax->SetRange(b1,b2);
  }

  //plot for sPlot with M0 known
  TCanvas *csP = new TCanvas("csP", "csP", 800, 400);
  csP->Divide(2);
  csP->cd(1);
  h[4]->SetLineColor(2);
  h[4]->SetLineWidth(1);
  h[4]->DrawCopy("E1");
  h[1]->SetLineColor(kGreen+1);
  h[1]->Draw("same");
  h[4]->SetLineColor(1);
  h[4]->SetLineWidth(2);
  h[4]->DrawCopy("hist,same");
  csP->cd(2);
  twoDim_mt_HT_Sig->Draw("colz,0");

  //plot for sPlot with M0 UNknown
  TCanvas *cesP = new TCanvas("cesP", "cesP", 800,400);
  cesP->Divide(2);
  cesP->cd(1);
  h[6]->SetLineColor(2);
  h[6]->SetLineWidth(1);
  h[6]->DrawCopy("E1");
  h[6]->SetLineColor(1);
  h[6]->SetLineWidth(2);
  h[6]->Draw("hist,same");
  cesP->cd(2);
  twoDim_mt_HT_Sig_esP->Draw("colz,0");  
}
